SLASH_TRADESKILLEXPORTER1 = "/tse"
SlashCmdList["TRADESKILLEXPORTER"] = function(msg)
	if msg == "help" then
		print("Tradeskill Exporter - Help")
		print("Type '/tse help' to show this message")
		print("Open a tradeskill window and type /tse to open the tradeskill exporter")
		print("Type '/tse csvlinks' to export only the link for CSV")
	else
		local craftName, craftRank, _ = GetCraftDisplaySkillLine()
		local tsName, tsRank, _ = GetTradeSkillLine()
		if(tsRank == 0 and craftRank == 0) then
			openTradeSkillExporterWindow('', craftRank, 'This will remove all your tradeskills from the database.', 0)
		else
			if(craftRank > 0) then
				local text = ''
				if msg == "csvlinks" then
				  text = '\n'
				end
				local recipyCount = 0
				for i = 1,GetNumCrafts() do
					local name, _, type, _, _, _, _ = GetCraftInfo(i)
					if (name and type ~= "header") then
						if msg == "csvlinks" then
							text = text .. '\n=HYPERLINK("https://classic.wowhead.com/spell=' .. getItemId("craft", i) .. '";"' .. name .. '")'
						else
							text = text .. encode64(name .. "(spell=" .. getItemId("craft", i) .. ")\n")
						end
						recipyCount = recipyCount + 1
					end
				end
				openTradeSkillExporterWindow(craftName, craftRank, text, recipyCount)
			elseif(tsRank > 0) then
				local text = ''
				local recipyCount = 0
				for i = 1,GetNumTradeSkills() do
					local name, type, _, _, _, _ = GetTradeSkillInfo(i)
					if (name and type ~= "header") then
						if msg == "csvlinks" then
							text = text .. '=HYPERLINK("https://classic.wowhead.com/item=' .. getItemId("trade", i) .. '";"' .. name .. '")\n'
						else
							text = text .. encode64(name .. "(item=" .. getItemId("trade", i) .. ")\n")
						end
						recipyCount = recipyCount + 1
					end
				end
				openTradeSkillExporterWindow(tsName, tsRank, text, recipyCount)
			end
		end 
	end
end

function openTradeSkillExporterWindow(tradeskillName, rank, text, recipyCount) 
	if not TradeSkillExporterWindow then
		createTradeSkillExporterWindow()
	end
	local playerName, realm = UnitName("player")
	local guildName, guildRankName, guildRankIndex = GetGuildInfo("player")
	local gameVersion = GetBuildInfo();
	local serverName = GetRealmName();
	if(guildName == nil) then
		guildName='-'
	end
	local editText = 
		"Player: " .. playerName .. "\n" ..
		"Guild: " .. guildName .. "\n" ..
		"Server: " .. serverName .. "\n"
	if(rank > 0) then
		TradeSkillExporterWindow.title:SetText(tradeskillName .. " skill " .. rank .. " - " .. recipyCount .. " recipies - Press CTRL-C to copy.")
		editText = editText .. "Tradeskill: " .. tradeskillName .. "\n"
	else
		TradeSkillExporterWindow.title:SetText("This will DELETE all tradeskills for " .. playerName .. " from the database - Press CTRL-C to copy.")
		editText = editText .. "This will DELETE all tradeskills for " .. playerName .. " from the database\n"
	end
	editText = editText .. "Export: " .. encode64(playerName .. "\n" .. guildName .. "\n" .. serverName .. "\n" .. tradeskillName .. "\n" .. gameVersion .. "\n") .. text
	TradeSkillExporterWindow.editBox:SetText(editText)
	TradeSkillExporterWindow.editBox:HighlightText()
	TradeSkillExporterWindow:Show()
end

function createTradeSkillExporterWindow()
	local frame = CreateFrame("Frame", "TradeSkillExporterWindow", UIParent, "BasicFrameTemplateWithInset")
	frame:SetSize(800, 600)
	frame:SetPoint("CENTER")
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata("HIGH")
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
	frame.title = frame:CreateFontString(nil, "OVERLAY")
	frame.title:SetFontObject("GameFontHighlight")
	frame.title:SetPoint("LEFT", frame.TitleBg, 5, 0)	
	frame.scrollFrame = CreateFrame("ScrollFrame", "TradeSkillExporterScrollFrame", TradeSkillExporterWindow	, "UIPanelScrollFrameTemplate")
	frame.scrollFrame:SetPoint("TOPLEFT", TradeSkillExporterWindow.InsetBg, "TOPLEFT", 8, -8)
	frame.scrollFrame:SetPoint("BOTTOMRIGHT", TradeSkillExporterWindow.InsetBg, "BOTTOMRIGHT", -3, 60)
	frame.scrollFrame.ScrollBar:SetPoint("TOPLEFT", frame.scrollFrame, "TOPRIGHT", -20, -22)
	frame.scrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", frame.scrollFrame, "BOTTOMRIGHT", -15, 22)
	frame.editBox = CreateFrame("EditBox", "TradeSkillExporterEditBox", TradeSkillExporterScrollFrame)
	frame.editBox:SetPoint("TOPLEFT", frame.scrollFrame, 5, -5)
	frame.editBox:SetFontObject(ChatFontNormal)
	frame.editBox:SetWidth(780)
	frame.editBox:SetAutoFocus(true)
	frame.editBox:SetMultiLine(true)
	frame.editBox:SetMaxLetters(99999)
	frame.editBox:SetScript("OnEscapePressed", function() frame:Hide() end)
	frame.scrollFrame:SetScrollChild(frame.editBox)	
end

function getItemId(prof_type, index)
  local itemLink, itemID
  if (prof_type == "trade") then
    itemLink = GetTradeSkillItemLink(index)
    if (not itemLink) then return end
    itemID = itemLink:match("item:(%d+)")
  elseif (prof_type == "craft") then
    itemLink = GetCraftItemLink(index)
    if (not itemLink) then return end
    itemID = itemLink:match("enchant:(%d+)")
  end

  return tonumber(itemID)

end

-- BASE 64 Codeing

local band, lshift, rshift = bit.band, bit.lshift, bit.rshift
local t = {}
local encode, decode = {}, { [strbyte("=")] = false }
for value = 0, 63 do
	local char = strsub('ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/', value+1, value+1)
	encode[value] = char
	decode[strbyte(char)] = value
end

function encode64(str)
	local j = 1
	for i = 1, strlen(str), 3 do
		local a, b, c = strbyte(str, i, i+2)
		t[j] = encode[rshift(a, 2)]
		t[j+1] = encode[band(lshift(a, 4) + rshift(b or 0, 4), 0x3F)]
		t[j+2] = b and encode[band(lshift(b, 2) + rshift(c or 0, 6), 0x3F)] or "="
		t[j+3] = c and encode[band(c, 0x3F)] or "="
		j = j + 4
	end
	return table.concat(t, "", 1, j-1)
end

function decode64(str)
	local j = 1
	assert(strlen(str) % 4 == 0, format("%s: invalid data length: %d", MAJOR, strlen(str)))
	for i = 1, strlen(str), 4 do
		local ba, bb, bc, bd = strbyte(str, i, i+3)
		local a, b, c, d = decode[ba], decode[bb], decode[bc], decode[bd]
		assert(a ~= nil, format("%s: invalid data at position %d: '%s'", MAJOR, i, ba))
		assert(b ~= nil, format("%s: invalid data at position %d: '%s'", MAJOR, i+1, bb))
		assert(c ~= nil, format("%s: invalid data at position %d: '%s'", MAJOR, i+2, bc))
		assert(d ~= nil, format("%s: invalid data at position %d: '%s'", MAJOR, i+3, bd))
		t[j] = strchar(lshift(a, 2) + rshift(b, 4))
		t[j+1] = c and strchar(band(lshift(b, 4) + rshift(c, 2), 0xFF)) or ""
		t[j+2] = d and strchar(band(lshift(c, 6) + d, 0xFF)) or ""
		j = j + 3
	end
	return table.concat(t, "", 1, j-1)
end
